#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include <string.h>
#include "SndfileWavUtil.h"

const  int Taps = 9;
const  int Points = 8;
const  int log_Points = 3;
static int newest;
static short int x[Taps] = {0,0,0,0,0,0,0,0,0};
static short int fir(short int);

// foreward declaration. This is defined in DFT.cpp
int DFT(int dir,int m,double *x1,double *y1);

// lifted from the bluespec code
static double h[Taps] =  {-0.0124,
			  0.0,
			  -0.0133,
			  0.0,
			  0.8181,
			  0.0,
			  -0.0133,
			  0.0,
			  -0.0124};

int
main (int argc, char * argv [])
{
  const char* inputWavFileName;
  const char* outputWavFileName;

  FILE *inputPcmFile;
  FILE *firPcmFile;
  FILE *outputPcmFile;

  short int sample;
  short int samples[Points];

  inputWavFileName = argv[1];
  outputWavFileName = argv[2];

  // Convert input wav to pcm
  generate_pcm(inputWavFileName, "input.pcm");

  inputPcmFile  = fopen("input.pcm", "r");
  firPcmFile    = fopen("fir.pcm", "w");

  assert(inputPcmFile);
  assert(firPcmFile);

  newest = 0;
  memset(x, 0, sizeof(x));

  while(fread(&sample, 2, 1, inputPcmFile)) {
    sample = fir(sample);
    assert(fwrite(&sample,2,1,firPcmFile));
  }

  fclose(inputPcmFile);
  fclose(firPcmFile);

  firPcmFile    = fopen("fir.pcm", "r");
  outputPcmFile = fopen("output.pcm", "w");

  assert(firPcmFile);
  assert(outputPcmFile);

  // we will do an Points point fft, and then undo it
  while(true) {

    int read = fread(samples, 2,Points,firPcmFile);

    if(read == 0)
      break;

    // pad out the 
    if(read<Points){
      for(int i = read; i < Points; i++)
	samples[i] = 0;
    }

    double dsamples[Points];
    double dimag[Points];

    // This shift is performed in the HW
    for(int i = 0; i < Points; i++){
      dsamples[i] = (double)(samples[i]>>log_Points);
      dimag[i] = 0.0;
    }

    DFT(1,Points,dsamples,dimag);
    DFT(-1,Points,dsamples,dimag);

    for(int i = 0; i < Points; i++){
      samples[i] = (int)dsamples[i];
    }

    if(read < Points){
      assert(fwrite(samples,2,read,outputPcmFile)==read);
      break;
    } else {
      assert(fwrite(samples,2,Points,outputPcmFile)==Points);
    }
  }

  fclose(firPcmFile);
  fclose(outputPcmFile);

  generate_wav("output.pcm", inputWavFileName, outputWavFileName);

}

short int
fir(short int sample)
{

  x[newest] = sample; 
  double y = 0;

  for (int k = 0; k < Taps; k++) { 

    short int bits = x[(newest+k)%Taps];
    double x = (double)bits;
    y += h[k] * x; 

  }

  newest = (newest == 0) ? (Taps-1) : (newest-1);

  short int rv = ((short int)y)&0x0000FFFF;
  return rv;
}

